<?php
/**
 * Created by PhpStorm.
 * User: Frédéric
 * Date: 21/06/2016
 * Time: 10:11
 */

namespace ByDedi\AdminBundle\Admin;

use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\EntityManagerInterface;
use Sonata\AdminBundle\Admin\AbstractAdmin;
use Sonata\AdminBundle\Admin\FieldDescriptionInterface;
use Sonata\AdminBundle\Admin\Pool;
use Sonata\AdminBundle\Route\RouteCollection;
use Sonata\AdminBundle\Show\ShowMapper;
use Sonata\DoctrineORMAdminBundle\Admin\FieldDescription;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

class BaseAdmin extends AbstractAdmin
{
    /**
     * @var EntityManagerInterface
     */
    protected $em;

    /**
     * @var ContainerInterface
     */
    protected $container;

    /**
     * @var array
     */
    protected $parentMenu = [];

    /**
     * @var string
     */
    protected $baseTranslationDomain = 'ByDediAdminBundle';

    /**
     * @var string
     */
    protected $translationDomain = 'ByDediAdminBundle';

    /**
     * @var array
     */
    public $tabsOptions = [
        'display' => 'fade' // fade | scroll
    ];

    /**
     * @var array
     */
    protected $translatableProperties = [];

    /**
     * @var array
     */
    protected $accessMapping = array(
        'clone' => 'CLONE'
    );

    /**
     * @var int
     */
    protected $maxPageLinks = 5;

    /** @var bool $supportsPreviewMode - allow preview button in form if set to true */
    public $supportsPreviewMode = false;

    /**
     * @var bool
     * wether validation buttons should be present or not
     */
    public $hasValidationButtons = true;

    /**
     * BaseAdmin constructor.
     * @param string $code
     * @param string $class
     * @param string $baseControllerName
     */
    public function __construct($code, $class, $baseControllerName)
    {
        parent::__construct($code, $class, $baseControllerName);
        unset($this->listModes['mosaic']);
    }

    /**
     * @param Pool $configurationPool
     */
    public function setConfigurationPool(Pool $configurationPool)
    {
        parent::setConfigurationPool($configurationPool);
        $this->container = $this->getConfigurationPool()->getContainer();
        $this->em = $this->getContainer()->get('doctrine.orm.entity_manager');
    }

    /**
     * @return ContainerInterface
     */
    public function getContainer()
    {
        return $this->container;
    }


    /**
     * {@inheritdoc}
     */
    public function attachAdminClass(FieldDescriptionInterface $fieldDescription)
    {
        if ($this->container->hasParameter('bydedi.default_admin')) {
            $defaultAdmins = $this->container->getParameter('bydedi.default_admin');
            $mapping = $fieldDescription->getAssociationMapping();
            if (isset($mapping['targetEntity']) && array_key_exists($mapping['targetEntity'], $defaultAdmins)) {
                $adminCode = $defaultAdmins[$mapping['targetEntity']];
                $fieldDescription->setOption('admin_code', $adminCode);
            }
        }

        return parent::attachAdminClass($fieldDescription);
    }


    /**
     * @param mixed $object
     */
    public function prePersist($object)
    {
        $this->hydrateTranslations($object);
        $this->collectionClearFix();
    }

    /**
     * @param mixed $object
     */
    public function postPersist($object)
    {
    }

    /**
     * @param mixed $object
     */
    public function preUpdate($object)
    {
        $this->hydrateTranslations($object);
        $this->collectionClearFix();
    }

    /**
     * @param mixed $object
     */
    public function postUpdate($object)
    {
    }

    /**
     * @param mixed $object
     */
    public function preRemove($object)
    {
    }

    /**
     * @param mixed $object
     */
    public function postRemove($object)
    {
    }

    /**
     * @param $object
     */
    public function hydrateTranslations(&$object)
    {
        $post = $this->getRequest()->request->get($this->getUniqid());
        $translationData = [];
        if (!empty($post)) {
            foreach ($post as $key => $values) {
                if (preg_match("#translations[0-9]*#", $key)) {
                    foreach ($values as $local => $translations) {
                        if (!array_key_exists($local, $translationData)) {
                            $translationData[$local] = [];
                        }
                        $translationData[$local] = array_merge($translations, $translationData[$local]);
                    }
                }
            }
        }
        if (!empty($translationData)) {
            foreach ($translationData as $local => $data) {
                foreach ($data as $property => $value) {
                    $method = 'set' . ucfirst($property);
                    if (is_callable([$object,$method])) {
                        $object->translate($local, false)->$method($value);
                    }
                }
            }
        }
        if (is_callable([$object,'mergeNewTranslations'])) {
            $object->mergeNewTranslations();
        }
    }

    /**+
     * @param array $fieldsParameters
     * @param array $options
     * @return array
     */
    protected function setTranslatableFieldsOptions($fieldsParameters = array(), $options = [])
    {
        if (empty($this->translatableProperties)) {
            $translationClass = $this->getClass() . 'Translation';
            if (class_exists($translationClass)) {
                $reflection = new \ReflectionClass(new $translationClass);
                $props = $reflection->getProperties();
                foreach ($props as $p) {
                    $this->translatableProperties[] = $p->name;
                }
            }
        }

        $merge = array_merge([
            "property_path" => "translations",
            "label" => false,
            "exclude_fields" => array_diff($this->translatableProperties, array_keys($fieldsParameters)),
            "fields" => $fieldsParameters
        ], $options);

        return is_array($merge) ? $merge : [];
    }

    /**
     * @param $object
     */
    public function collectionClearFix()
    {
        $form = $this->getForm();
        $children = $form->all();

        foreach ($children as $childForm) {
            $data = $childForm->getData();
            if ($data instanceof Collection) {
                $proxies = $childForm->all();
                foreach ($proxies as $proxy) {
                    $entity = $proxy->getData();
                    if (!$data->contains($entity)) {
                        $this->getModelManager()->delete($entity);
                    }
                }
            }
        }
    }

    /**
     * @param $object
     * @param $data
     * @return bool
     */
    protected function checkSlug($object, $data)
    {
        if (method_exists($object, 'getSluggableFields') && is_callable([$object,'getSluggableFields'])) {
            $sluggableFields = $object->getSluggableFields();
            $slugOk = false;
            foreach ($data as $k => $d) {
                if (in_array($k, $sluggableFields) && !empty($d)) {
                    $slugOk = true;
                }
            }
            return $slugOk;
        }
        return true;
    }

    /**
     * @param string $name
     * @return mixed|null|string
     */
    public function getTemplate($name)
    {
        switch ($name) {
            case 'sublist':
                return 'AdminBundle:admin/CRUD:sublist.html.twig';
                break;
            case 'advanced_search':
                return 'AdminBundle:admin/Search:advanced_search.html.twig';
                break;
            case 'send_mail':
                return 'AdminBundle:admin/CRUD:send_mail.html.twig';
                break;
            default:
                return parent::getTemplate($name);
                break;
        }
    }

    /**
     * @param RouteCollection $collection
     */
    protected function configureRoutes(RouteCollection $collection)
    {
        $collection->add('history', $this->getRouterIdParameter().'/history');
        $collection->add('clone', $this->getRouterIdParameter().'/clone');
        $collection->add('sublist', 'sublist/');
        $collection->add('import', 'import/');
        $collection->add('custom-export', 'custom-export/');
        $collection->remove('acl');
    }

    /**
     * @return \Exporter\Source\SourceIteratorInterface
     */
    public function getDataSourceIterator()
    {
        $datasourceit = parent::getDataSourceIterator();
        $datasourceit->setDateTimeFormat('Y-m-d'); //change this to suit your needs
        return $datasourceit;
    }

    /**
     * @param string $action
     * @param mixed  $object
     *
     * @return array
     */
    public function getActionButtons($action, $object = null)
    {
        $list = parent::getActionButtons($action, $object);

        if (isset($list['create'])) {
            $list['create']['template'] = 'AdminBundle:admin/Button:create_button.html.twig';
        }
        if (isset($list['edit'])) {
            $list['edit']['template'] = 'AdminBundle:admin/Button:edit_button.html.twig';
        }
        if (isset($list['show'])) {
            $list['show']['template'] = 'AdminBundle:admin/Button:show_button.html.twig';
        }
        if (isset($list['history'])) {
            $list['history']['template'] = 'AdminBundle:admin/Button:history_button.html.twig';
        }
        if (isset($list['acl'])) {
            $list['acl']['template'] = 'AdminBundle:admin/Button:acl_button.html.twig';
        }
        if (isset($list['list'])) {
            $list['list']['template'] = 'AdminBundle:admin/Button:list_button.html.twig';
        }

        return $list;
    }

    /**
     * @return array
     */
    public function getDefaultListActionParams()
    {
        return [
            'actions' => [
                'edit' => [
                    'template' => 'AdminBundle:admin/CRUD:list__action_edit.html.twig'
                ],
                'delete' => [
                    'template' => 'AdminBundle:admin/CRUD:list__action_delete.html.twig'
                ]
            ],
            'template' => 'AdminBundle:admin/CRUD:list__action.html.twig'
        ];
    }

    /**
     * @param $queryBuilder
     * @param $alias
     * @param $field
     * @param $value
     * @return bool|void
     */
    public function getDateFilterCallback($queryBuilder, $alias, $field, $value)
    {
        if (!$value['value']) {
            return;
        }

        if ($value['value'] instanceof \DateTime) {
            $value['value'] = $value['value']->format('d-m-Y');
        }
        $time = strtotime(str_replace('/', '-', $value['value']));
        $inputValue = date('Y-m-d', $time);

        $queryBuilder->andWhere("DATE($alias.createdAt) = :CreatedAt");
        $queryBuilder->setParameter('CreatedAt', $inputValue);

        return true;
    }

    /**
     * @param $type
     * @return string
     */
    protected function getTemplateType($type)
    {
        switch ($type) {
            case 'image':
                return 'AdminBundle:admin/CRUD:image_list.html.twig';
            case 'list_file':
                return 'AdminBundle:admin/CRUD:file_list.html.twig';
            case 'list_order_status':
                return 'AdminBundle:admin/CRUD:list__order_status.html.twig';
            case 'list_relation_link':
                return 'AdminBundle:admin/CRUD:relation_link_list.html.twig';
            case 'file_input_help':
                return 'AdminBundle:admin/Form:file_input_help.html.twig';
            case 'browse_select_button':
                return 'AdminBundle:admin/CRUD:list__action_browse_select.html.twig';
            case 'upload_button':
                return 'AdminBundle:admin/Button:upload_button.html.twig';
            case 'autocomplete_field':
                return 'AdminBundle:admin/Form/:autocomplete.html.twig';
            case 'model_preview':
                return 'AdminBundle:admin/Form/:model.html.twig';
            case 'advanced_search':
                return 'AdminBundle:admin/Form/:advanced_search.html.twig';
            default:
                return '';
                break;
        }
    }

    /**
     * @return array
     */
    public function getParentMenu()
    {
        return $this->parentMenu;
    }

    /**
     * @param $template
     * @param array $vars
     * @return string
     */
    public function renderTemplate($template, $vars = [])
    {
        $templating = $this->getContainer()->get('templating');
        return $templating->render($template, array_merge(['subject' => $this->getSubject()], $vars));
    }

    /**
     * @return array
     */
    public function getExportFields()
    {
        $exportFields = parent::getExportFields();
        $list = $this->getList();
        $listFields = array();
        foreach ($list->getElements() as $key => $field) {
            if (!in_array($key, array('batch','_action'))) {
                $listFields[] = $key;
            }
        }

        return $listFields;
    }

    /**
     * @return array
     */
    public function getListImportActions()
    {
        return array();
    }

    /**
     * @return array
     */
    public function getFormImportActions()
    {
        return array();
    }

    /**
     * @return array
     */
    public function getExportActions()
    {
        return array();
    }

    /**
     * @return array
     */
    public function getFormExportActions()
    {
        return array();
    }

    /**
     * @return array
     */
    protected function configureCustomRoutes()
    {
        return array();
    }

    /**
     * @param $name
     * @throws RouteNotFoundException
     * @return mixed
     */
    public function getCustomRoute($name)
    {
        $routes = $this->configureCustomRoutes();
        if (isset($routes[$name]['route']) || array_key_exists($name, $routes)) {
            return $routes[$name]['route'];
        }
        throw new RouteNotFoundException('Custom admin route '.$name.' does not exists');
    }

    /**
     * @param $name
     *
     * @return bool
     */
    public function hasCustomRoute($name)
    {
        try {
            $this->getCustomRoute($name);
            return true;
        } catch (RouteNotFoundException $e) {
        }
        return false;
    }

    /**
     * @param ShowMapper $showMapper
     * @throws \RuntimeException
     */
    protected function configureShowFields(ShowMapper $showMapper)
    {
        $formFields = $this->getFormFieldDescriptions();

        if ($formFields !== null) {
            /** @var FieldDescription $field */
            foreach ($formFields as $fieldName => $field) {
                $fieldType = $field->getType();
                if ($fieldName === 'history' || false !== strpos('preview', $fieldType)) {
                    continue;
                }
                $showMapper->add($fieldName, $field->getType(), [
                    'label' => $field->getLabel(),
                ]);
            }
        }

        $showMapper->end();
    }


    /**
     * @param $queryBuilder \Doctrine\ORM\QueryBuilder
     */
    public function checkAliasExists($queryBuilder, $alias)
    {
        $joinDqlParts = $queryBuilder->getDQLParts()['join'];

        /* @var $join \Doctrine\ORM\Query\Expr\Join */
        foreach ($joinDqlParts as $joins) {
            foreach ($joins as $join) {
                if ($join->getAlias() === $alias) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @return bool
     */
    protected function isUpdate()
    {
        $class = $this->getClass();
        return $this->getSubject() instanceof $class && $this->getSubject()->getId();
    }

    /**
     * @return bool
     */
    protected function isEmbed()
    {
        $request = $this->getRequest();
        $controller = $request->attributes->get('_controller');
        $action = explode('::', $controller)[1];
        if (stripos($action, 'sublistAction') !== false) {
            return true;
        }

        $class = $this->getClass();
        return $this->getRoot()->getClass() != $class;
    }

    /**
     * @return mixed|null
     */
    protected function getParentSubject()
    {
        if ($this->hasParentFieldDescription()) {
            return $this->getParentFieldDescription()->getAdmin()->getSubject();
        }
        return null;
    }
}
